@@ -19,9 +19,9 @@
 
 """Provides a SpeechServer factory for gnome-speech drivers."""
 
-__id__        = "$Id: gnomespeechfactory.py,v 1.51 2006/07/28 19:49:30 wwalker Exp $"
-__version__   = "$Revision: 1.51 $"
-__date__      = "$Date: 2006/07/28 19:49:30 $"
+__id__        = "$Id: gnomespeechfactory.py 1388 2006-08-30 12:35:15Z wwalker $"
+__version__   = "$Revision: 1388 $"
+__date__      = "$Date: 2006-08-30 08:35:15 -0400 (Wed, 30 Aug 2006) $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -216,15 +216,20 @@ class SpeechServer(speechserver.SpeechServer):
         if not gserver:
             return None
 
+        server = None
         try:
             server = SpeechServer.__createServer(gserver.iid)
         except:
+            debug.printException(debug.LEVEL_WARNING)
+
+        if not server:
             for s in gservers:
                 try:
                     server = SpeechServer.__createServer(s.iid)
+                    if server:
+                        break
                 except:
                     debug.printException(debug.LEVEL_WARNING)
-                    pass
 
         return server
 
