@@ -20,9 +20,9 @@
 """Manages the magnifier for orca.  [[[TODO: WDW - this is very very
 early in development.  One might even say it is pre-prototype.]]]"""
 
-__id__        = "$Id: mag.py,v 1.29 2006/07/28 17:33:16 wwalker Exp $"
-__version__   = "$Revision: 1.29 $"
-__date__      = "$Date: 2006/07/28 17:33:16 $"
+__id__        = "$Id: mag.py 1339 2006-08-18 17:39:17Z wwalker $"
+__version__   = "$Revision: 1339 $"
+__date__      = "$Date: 2006-08-18 13:39:17 -0400 (Fri, 18 Aug 2006) $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -284,6 +284,16 @@ def applySettings():
 
     _magnifier.clearAllZoomRegions()
 
+    try:
+        _magnifier.TargetDisplay = settings.magnifierTargetDisplay
+    except:
+        pass
+
+    try:
+        _magnifier.SourceDisplay = settings.magnifierSourceDisplay
+    except:
+        pass
+
     magnifierPBag = _magnifier.getProperties()
 
     try:
@@ -315,7 +325,7 @@ def applySettings():
             magnifierPBag, "cursor-size", 0)
 
     bonobo.pbclient_set_string(magnifierPBag, "cursor-set", "default")
-    
+
     # Convert the colorPreference string to something we can use.
     # The main issue here is that the color preferences are saved
     # as 4 byte values per color.  We only need 2 bytes, so we
@@ -390,7 +400,7 @@ def applySettings():
         _pollMouseDisabled = True
     except:
         _pollMouseDisabled = False
-        
+
     _zoomer.setMagFactor(settings.magZoomFactor, settings.magZoomFactor)
 
     bonobo.pbclient_set_boolean(
