@@ -22,8 +22,13 @@
 A speech server (class SpeechServer) provides the ability to tell the
 machine to speak.  Each speech server provides a set of known
 voices (identified by name) which can be combined with various
-attributes to create aural style sheets.
-"""
+attributes to create aural style sheets."""
+
+__id__        = "$Id: speechserver.py 1036 2006-06-10 00:54:48Z wwalker $"
+__version__   = "$Revision: 1036 $"
+__date__      = "$Date: 2006-06-09 20:54:48 -0400 (Fri, 09 Jun 2006) $"
+__copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
+__license__   = "LGPL"
 
 import debug
 
@@ -195,6 +200,10 @@ class SpeechServer:
         """
         pass
 
+    def isSpeaking(self):
+	""""Returns True if the system is currently speaking."""
+	return False
+
     def sayAll(self, utteranceIterator, progressCallback):
         """Iterates through the given utteranceIterator, speaking
         each utterance one at a time.  Subclasses may postpone
@@ -224,6 +233,16 @@ class SpeechServer:
         """
         pass
 
+    def increaseSpeechPitch(self, step=0.5):
+        """Increases the speech pitch.
+        """
+        pass
+
+    def decreaseSpeechPitch(self, step=0.5):
+        """Decreases the speech pitch.
+        """
+        pass
+
     def stop(self):
         """Stops ongoing speech and flushes the queue."""
         pass
