@@ -19,9 +19,9 @@
 
 """Provides various utility functions for Orca."""
 
-__id__        = "$Id: util.py,v 1.27 2006/07/21 14:58:27 wwalker Exp $"
-__version__   = "$Revision: 1.27 $"
-__date__      = "$Date: 2006/07/21 14:58:27 $"
+__id__        = "$Id: util.py 1363 2006-08-23 12:30:59Z oserb $"
+__version__   = "$Revision: 1363 $"
+__date__      = "$Date: 2006-08-23 08:30:59 -0400 (Wed, 23 Aug 2006) $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -40,7 +40,7 @@ import string
 import atspi
 import debug
 import input_event
-import orca
+import orca_state
 import rolenames
 import settings
 import speech
@@ -48,6 +48,55 @@ import speechserver
 
 from orca_i18n import _ # for gettext support
 
+def isSameObject(obj1, obj2):
+    if (obj1 == obj2):
+        return True
+    elif (not obj1) or (not obj2):
+        return False
+
+    try:
+	if obj1.name != obj2.name:
+	    return False
+	
+	# When we're looking at children of objects that manage 
+	# their descendants, we will often get different objects 
+	# that point to the same logical child.  We want to be able 
+	# to determine if two objects are in fact pointing to the same child.
+	# If we cannot do so easily (i.e., object equivalence), we examine 
+	# the hierarchy and the object index at each level.
+	#
+	parent1 = obj1
+	parent2 = obj2
+	while (parent1 and parent2 and \
+		parent1.state.count(atspi.Accessibility.STATE_TRANSIENT) and \
+		parent2.state.count(atspi.Accessibility.STATE_TRANSIENT)):
+	    if parent1.index != parent2.index:
+		return False
+	    parent1 = parent1.parent
+	    parent2 = parent2.parent
+	if parent1 and parent2 and parent1 == parent2:
+	    return True
+    except:
+        pass
+
+    # In java applications, TRANSIENT state is missing for tree items
+    # (fix for bug #352250)
+    #
+    try:
+	parent1 = obj1
+	parent2 = obj2
+	while parent1 and parent2 and \
+		parent1.role == rolenames.ROLE_LABEL and \
+		parent2.role == rolenames.ROLE_LABEL:
+	    parent1 = parent1.parent
+	    parent2 = parent2.parent
+	if parent1 and parent2 and parent1 == parent2:
+	    return True
+    except:
+	pass
+    
+    return False
+
 def appendString(text, newText, delimiter=" "):
     """Appends the newText to the given text with the delimiter in between
     and returns the new string.  Edge cases, such as no initial text or
@@ -60,6 +109,57 @@ def appendString(text, newText, delimiter=" "):
     else:
         return newText
 
+def __hasLabelForRelation(label):
+    """Check if label has a LABEL_FOR relation
+
+    Arguments:
+    - label: the label in question
+
+    Returns TRUE if label has a LABEL_FOR relation.
+    """
+    if (not label) or (label.role != rolenames.ROLE_LABEL):
+        return False
+
+    relations = label.relations
+
+    for relation in relations:
+        if relation.getRelationType() \
+               == atspi.Accessibility.RELATION_LABEL_FOR:
+            return True
+
+    return False
+
+
+def __isLabeling(label, object):
+    """Check if label is connected via  LABEL_FOR relation with object
+
+    Arguments:
+    - object: the object in question
+    - labeled: the label in question
+
+    Returns TRUE if label has a relation LABEL_FOR for object.
+    """
+
+    if (not object) \
+       or (not label) \
+       or (label.role != rolenames.ROLE_LABEL):
+        return False
+
+    relations = label.relations
+    if not relations:
+        return False
+
+    for relation in relations:
+        if relation.getRelationType() \
+               == atspi.Accessibility.RELATION_LABEL_FOR:
+
+            for i in range(0, relation.getNTargets()):
+                target = atspi.Accessible.makeAccessible(relation.getTarget(i))
+                if target == object:
+                    return True
+
+    return False
+
 def getDisplayedLabel(object):
     """If there is an object labelling the given object, return the
     text being displayed for the object labelling this object.
@@ -103,58 +203,44 @@ def getDisplayedLabel(object):
     # other is the container for the grouped objects.  When we detect
     # this, we add the label to the overall context.
     #
-    # We are also looking for PANELs with two children, because we
-    # discovered similer cases for PANELS in java applications.  The
-    # java application that includes this behavior is FileChooserDemo.
-    # Choose Show FileChooser button, a new dialog appears. In this
-    # dialog, the issue presented is for "Files of Type" combo
-    # box. Moving to this combo, Orca speaks only the selected item
-    # from combo, "Files of Type" label is not spoken.]]]
+    # We are also looking for objects which have a PANEL or a FILLER as
+    # parent, and its parent has more children. Through these children,
+    # a potential label with LABEL_FOR relation may exists. We want to
+    # present this label.
+    # This case can be seen in FileChooserDemo application, in Open dialog
+    # window, the line with "Look In" label, a combobox and some presentation
+    # buttons.]]]
     #
-    if (not label) \
-        and ((object.role == rolenames.ROLE_FILLER) \
-             or (object.role == rolenames.ROLE_PANEL)) \
-        and (object.childCount == 2):
+    if not label:
 
-        potentialLabel = object.child(0)
-        labeledObject = object.child(1)
+        potentialLabel = None
+        useLabel = False
+        if ((object.role == rolenames.ROLE_FILLER) \
+                or (object.role == rolenames.ROLE_PANEL)) \
+            and (object.childCount == 2):
 
-        if potentialLabel and labeledObject \
-            and (potentialLabel.role == rolenames.ROLE_LABEL):
+            potentialLabel = object.child(0)
+            secondChild = object.child(1)
+            useLabel = potentialLabel.role == rolenames.ROLE_LABEL \
+                    and ((secondChild.role == rolenames.ROLE_FILLER) \
+                            or (secondChild.role == rolenames.ROLE_PANEL)) \
+                    and not __hasLabelForRelation(potentialLabel)
+        else:
+            parent = object.parent
+            if parent and \
+                ((parent.role == rolenames.ROLE_FILLER) \
+                        or (parent.role == rolenames.ROLE_PANEL)):
+                for i in range (0, parent.childCount):
+                    try:
+                        potentialLabel = parent.child(i)
+                        useLabel = __isLabeling(potentialLabel, object)
+                        if useLabel:
+                            break
+                    except:
+                        pass
 
-            # If the label is labeling something, we will ignore it
-            # because we're likely to pick it up via some other means.
-            # An example here is the "Shade transparent or image background:"
-            # label in gnome-terminal's "Effects" customization tab.
-            #
-            useLabel = True
-            for relation in potentialLabel.relations:
-                if relation.getRelationType() \
-                    == atspi.Accessibility.RELATION_LABEL_FOR:
-
-                    # If potentialLabel is labeling labeledObject, we
-                    # check to see if the labeledObject has the
-                    # LABELLED_BY relation.  If the labeledObject
-                    # doensn't have a LABELLED_BY relation, then the
-                    # potentialLabel is likely not to be picked up via
-                    # other means, so we use this as a label for the
-                    # FILLER or PANEL.
-                    #
-                    useLabel = False
-                    for i in range(0, relation.getNTargets()):
-                        target = atspi.Accessible.makeAccessible(
-                            relation.getTarget(i))
-                        if target == labeledObject:
-                            useLabel = True
-                            for rel in labeledObject.relations:
-                                if rel.getRelationType() \
-                                   == atspi.Accessibility.RELATION_LABELLED_BY:
-                                    useLabel = False
-                                    break
-                        break
-                    break
-            if useLabel:
-                label = potentialLabel.name
+        if useLabel and potentialLabel:
+            label = potentialLabel.name
 
     return label
 
@@ -200,8 +286,11 @@ def __getDisplayedTextInComboBox(combo):
         selectedItem = None
         comboSelection = combo.selection
         if comboSelection and comboSelection.nSelectedChildren > 0:
-            selectedItem = atspi.Accessible.makeAccessible(
-                comboSelection.getSelectedChild(0))
+            try:
+                selectedItem = atspi.Accessible.makeAccessible(
+                    comboSelection.getSelectedChild(0))
+            except:
+                pass
         if selectedItem:
             displayedText = getDisplayedText(selectedItem)
             #print "SELECTEDITEM", displayedText
@@ -584,7 +673,7 @@ def getNodeLevel(obj):
     if not obj:
         return -1
 
-    level = -1
+    nodes = []
     node = obj
     done = False
     while not done:
@@ -593,13 +682,25 @@ def getNodeLevel(obj):
         for relation in relations:
             if relation.getRelationType() \
                    == atspi.Accessibility.RELATION_NODE_CHILD_OF:
-                level += 1
                 node = atspi.Accessible.makeAccessible(relation.getTarget(0))
                 break
-        done = not node
-        debug.println(debug.LEVEL_FINEST, "util.getNodeLevel %d" % level)
 
-    return level
+        # We want to avoid situations where something gives us an
+        # infinite cycle of nodes.  Bon Echo has been seen to do
+        # this (see bug 351847).
+        #
+        if (len(nodes) > 100) or nodes.count(node):
+            debug.println(debug.LEVEL_WARNING,
+                          "util.getNodeLevel detected a cycle!!!")
+            done = True
+        elif node:
+            nodes.append(node)
+            debug.println(debug.LEVEL_FINEST,
+                          "util.getNodeLevel %d" % len(nodes))
+        else:
+            done = True
+
+    return len(nodes) - 1
 
 def getAcceleratorAndShortcut(obj):
     """Gets the accelerator string (and possibly shortcut) for the given
@@ -670,7 +771,7 @@ def getKnownApplications():
             debug.printException(debug.LEVEL_FINEST)
 
     debug.println(debug.LEVEL_FINEST,
-                  "...orca._buildAppList")
+                  "...util.getKnownApplications")
 
     return apps
 
@@ -903,7 +1004,8 @@ def isInActiveApp(obj):
     if not obj:
         return False
     else:
-        return orca.locusOfFocus and (orca.locusOfFocus.app == obj.app)
+        return orca_state.locusOfFocus \
+               and (orca_state.locusOfFocus.app == obj.app)
 
 def findActiveWindow():
     """Traverses the list of known apps looking for one who has an
@@ -917,10 +1019,13 @@ def findActiveWindow():
     apps = getKnownApplications()
     for app in apps:
         for i in range(0, app.childCount):
-            state = app.child(i).state
-            if state.count(atspi.Accessibility.STATE_ACTIVE) > 0:
-                window = app.child(i)
-                break
+            try:
+                state = app.child(i).state
+                if state.count(atspi.Accessibility.STATE_ACTIVE) > 0:
+                    window = app.child(i)
+                    break
+            except:
+                debug.printException(debug.LEVEL_FINEST)
 
     return window
 
@@ -949,7 +1054,7 @@ def drawOutline(x, y, width, height, erasePrevious=True):
 
         if not _display:
             debug.println(debug.LEVEL_SEVERE,
-                          "orca.drawOutline could not open display.")
+                          "util.drawOutline could not open display.")
             return
 
     screen = _display.get_default_screen()
@@ -1051,8 +1156,13 @@ def speakTextSelectionState(obj, startOffset, endOffset):
     # Control-Shift-Home: speak "document selected to cursor position".
     # Control-Shift-End:  speak "document selected from cursor position".
     #
-    eventStr = orca.lastInputEvent.event_string
-    mods = orca.lastInputEvent.modifiers
+    if isinstance(orca_state.lastInputEvent, input_event.KeyboardEvent):
+        eventStr = orca_state.lastInputEvent.event_string
+        mods = orca_state.lastInputEvent.modifiers
+    else:
+        eventStr = None
+        mods = 0
+
     isControlKey = mods & (1 << atspi.Accessibility.MODIFIER_CONTROL)
     isShiftKey = mods & (1 << atspi.Accessibility.MODIFIER_SHIFT)
 
@@ -1133,3 +1243,38 @@ def speakTextSelectionState(obj, startOffset, endOffset):
     obj.lastSelections = []
     for i in range(0, obj.text.getNSelections()):
         obj.lastSelections.append(obj.text.getSelection(i))
+
+# The following code has been borrowed from the following URL:
+#
+# http://www.dalkescientific.com/writings/diary/archive/2005/04/20/tracing_python_code.html
+#
+import linecache
+
+def traceit(frame, event, arg):
+    """Line tracing utility to output all lines as they are executed by
+    the interpreter.  This is to be used by sys.settrace and is for
+    debugging purposes.
+
+    Arguments:
+    - frame: is the current stack frame
+    - event: 'call', 'line', 'return', 'exception', 'c_call', 'c_return',
+             or 'c_exception'
+    - arg:   depends on the event type (see docs for sys.settrace)
+    """
+
+    if event == "line":
+        lineno = frame.f_lineno
+        filename = frame.f_globals["__file__"]
+        if (filename.endswith(".pyc") or
+            filename.endswith(".pyo")):
+            filename = filename[:-1]
+        name = frame.f_globals["__name__"]
+        if name == "gettext" \
+           or name == "locale" \
+           or name == "posixpath" \
+           or name == "UserDict":
+            return traceit
+        line = linecache.getline(filename, lineno)
+        debug.println(debug.LEVEL_ALL,
+                      "TRACE %s:%s: %s" % (name, lineno, line.rstrip()))
+    return traceit
